#include "lib/Compatibility.frag"

#define USE_VIEW_POSITION
#define USE_LIGHTS

#define FEATURE_TEXTURED
#define FEATURE_ALPHA_MASKED
#define FEATURE_VERTEX_COLORS
#define FEATURE_NORMAL_MAPPING
#define FEATURE_WITH_EMISSIVE
#define FEATURE_OCCLUSION_TEXTURE
#define FEATURE_GLOBAL_ILLUMINATION
#define FEATURE_TONEMAPPING
#define FEATURE_SHADOW_NORMAL_OFFSET_SCALE_BY_SHADOW_DEPTH
#define FEATURE_SHADOW_NORMAL_OFFSET_UV_ONLY
#define FEATURE_SHADOW_NORMAL_OFFSET_SLOPE_SCALE

#ifdef NORMAL_MAPPING
#define TEXTURED
#endif
#ifdef OCCLUSION_TEXTURE
#define TEXTURED
#endif
#ifdef GLOBAL_ILLUMINATION
#define TEXTURED
#endif

#define USE_POSITION_WORLD
#define USE_NORMAL
#define USE_MATERIAL_ID
#ifdef TEXTURED
#define USE_TEXTURE_COORDS
#endif
#ifdef NORMAL_MAPPING
#define USE_TANGENT
#endif

#ifdef VERTEX_COLORS
#define USE_COLOR
#endif

#if NUM_SHADOWS > 0
#define USE_POSITION_VIEW
#endif

#include "lib/Uniforms.glsl"
#include "lib/Inputs.frag"
#include "lib/Math.glsl"
#include "lib/Color.glsl"

#if NUM_LIGHTS > 0
#include "lib/Quaternion.glsl"
#include "lib/Lights.frag"
#endif

#ifdef TEXTURED
#include "lib/Textures.frag"
#endif
#include "lib/Surface.frag"
#include "lib/Materials.frag"

#ifdef GLOBAL_ILLUMINATION
#include "lib/CoordinateSystems.glsl"
#include "lib/GI.frag"
#endif

struct Material {
    lowp vec4 albedoColor;
#ifdef WITH_EMISSIVE
    lowp vec4 emissiveColor;
#endif
    lowp float metallicFactor;
    lowp float roughnessFactor;
#ifdef TEXTURED
    mediump uint albedoTexture;
    mediump uint roughnessMetallicTexture;
#ifdef WITH_EMISSIVE
    mediump uint emissiveTexture;
#endif
#ifdef NORMAL_MAPPING
    mediump uint normalTexture;
#endif
#ifdef OCCLUSION_TEXTURE
    mediump uint occlusionTexture;
    lowp float occlusionFactor;
#endif
#ifdef GLOBAL_ILLUMINATION // Should be moved out of material.
    mediump uint irradianceProbeTexture;
    mediump uint specularProbeTexture;
#endif
#endif
};

Material decodeMaterial(uint matIndex) {
    {{decoder}}
    return mat;
}

vec3 fresnelSchlick(float cosTheta, vec3 F0) {
    return F0 + (vec3(1.0) - F0)*pow(1.0 - cosTheta, 5.0);
}

float distributionGGX(vec3 N, vec3 H, float roughness) {
    float a = roughness*roughness;
    float a2 = a*a;
    float NdotH = max(dot(N, H), 0.0);
    float NdotH2 = NdotH*NdotH;

    float num = a2;
    float denom = (NdotH2*(a2 - 1.0) + 1.0);
    denom = PI*denom*denom;

    return num/denom;
}

float geometrySchlickGGX(float NdotV, float roughness) {
    float a = roughness*roughness;
    float r = (a + 1.0);
    float k = (r*r) / 8.0;

    float num = NdotV;
    float denom = NdotV*(1.0 - k) + k;

    return num/denom;
}

float geometrySmith(vec3 N, vec3 V, vec3 L, float roughness) {
    float NdotV = max(dot(N, V), 0.0);
    float NdotL = max(dot(N, L), 0.0);
    float ggx2 = geometrySchlickGGX(NdotV, roughness);
    float ggx1 = geometrySchlickGGX(NdotL, roughness);

    return ggx1*ggx2;
}

vec3 physicalBrdf(mediump vec3 L, mediump vec3 normal, mediump vec3 view, mediump vec3 albedo, mediump float metallic, mediump float roughness, mediump vec3 f0) {
    vec3 halfVec = normalize(view + L);

    float NDF = distributionGGX(normal, halfVec, roughness);
    float G = geometrySmith(normal, view, L, roughness);
    vec3 f = fresnelSchlick(max(dot(halfVec, view), 0.0), f0);

    vec3 kD = (vec3(1.0) - f)*(1.0 - metallic);

    vec3 numerator = NDF*G*f;
    float NdotL = max(dot(normal, L), 0.0);
    float denominator = 4.0*max(dot(normal, view), 0.0)*NdotL;
    vec3 specular = numerator/max(denominator, 0.001);

    return (kD*albedo/PI + specular)*NdotL;
}

void main() {
    #ifdef TEXTURED
    alphaMask(fragMaterialId, fragTextureCoords);
    #endif

    Material mat = decodeMaterial(fragMaterialId);

    lowp vec4 albedo =
        #ifdef VERTEX_COLORS
        fragColor*
        #endif
        mat.albedoColor;

    #ifdef TEXTURED
    if(mat.albedoTexture > 0u) {
        albedo *= textureAtlas(mat.albedoTexture, fragTextureCoords);
    }
    #endif
    albedo = srgbToLinear(albedo);

    float roughness = mat.roughnessFactor;
    float metallic = mat.metallicFactor;
    #ifdef TEXTURED
    if(mat.roughnessMetallicTexture > 0u) {
        lowp vec4 rm = textureAtlas(mat.roughnessMetallicTexture, fragTextureCoords);
        roughness *= rm.g;
        metallic *= rm.b;
    }
    #endif

    vec3 f0 = mix(vec3(0.04), albedo.rgb, metallic);

    /* Ambient color */
    lowp float ao = 1.0;
    #ifdef OCCLUSION_TEXTURE
    if(mat.occlusionTexture > 0u) {
        float occlusion = textureAtlas(mat.occlusionTexture, fragTextureCoords).r;
        ao = mix(1.0, occlusion, mat.occlusionFactor);
    }
    #endif

    /* Normal */
    #ifdef NORMAL_MAPPING
    SurfaceData surface = computeSurfaceData(fragNormal, fragTangent);
    mediump vec3 normal = normalMapping(surface, mat.normalTexture);
    #else
    SurfaceData surface = computeSurfaceData(fragNormal);
    mediump vec3 normal = surface.normal;
    #endif

    vec3 view = normalize(viewPositionWorld - fragPositionWorld);

    /* Contains the indirect specular component, i.e., specular reuslting
     * from Global Illumiation.
     */
    vec3 indirectDiffuse = vec3(0.0);
    vec3 indirectSpecular = vec3(0.0);

    #ifdef GLOBAL_ILLUMINATION
    /* TODO: refactor with NdotV from geometrySmith */
    float NdotV = max(dot(normal, view), 0.0);
    vec3 reflectWS = normalize(reflect(- view, normal));
    computeEnvContribution(mat.irradianceProbeTexture, mat.specularProbeTexture,
        normal, reflectWS, NdotV, roughness, metallic, f0, indirectDiffuse, indirectSpecular);
    #else
    indirectDiffuse = vec3(0.03);
    #endif
    indirectDiffuse *= albedo.rgb;

    vec3 col = (indirectDiffuse + indirectSpecular) * ao;

    #if NUM_LIGHTS > 0

    lowp uint i = 0u;
    for(; i < numPointLights; ++i) {
        mediump vec4 lightData = lightColors[i];
        /* dot product of mediump vec3 can be NaN for distances > 128 */
        highp vec3 lightPos = lightPositionsWorld[i];
        highp vec3 lightDirAccurate = lightPos - fragPositionWorld;
        mediump float distSq = dot(lightDirAccurate, lightDirAccurate);
        mediump float attenuation = distanceAttenuation(distSq, lightData.a);
        if(attenuation < 0.001)
            continue;

        mediump vec3 lightDir = lightDirAccurate;
        lightDir *= inversesqrt(distSq);

        mediump vec3 value = physicalBrdf(lightDir, normal, view, albedo.rgb, metallic, roughness, f0);

        float shadow = 1.0;
        #if NUM_SHADOWS > 0
        /* Shadows */
        bool shadowsEnabled = bool(lightParameters[i].z);
        if(shadowsEnabled) {
            int shadowIndex = int(lightParameters[i].w) + int(dot(lightDir, lightDirectionsWorld[i]) < 0.0);
            shadow = sampleShadowParaboloid(shadowIndex);
        }
        #endif

        col += attenuation*value*lightData.rgb*shadow;
    }

    lowp uint endSpotLights = numPointLights + numSpotLights;
    for(; i < endSpotLights; ++i) {
        mediump vec4 lightData = lightColors[i];
        /* dot product of mediump vec3 can be NaN for distances > 128 */
        highp vec3 lightPos = lightPositionsWorld[i];
        highp vec3 lightDirAccurate = lightPos - fragPositionWorld;
        mediump float distSq = dot(lightDirAccurate, lightDirAccurate);
        mediump float attenuation = distanceAttenuation(distSq, lightData.a);

        if(attenuation < 0.001)
            continue;

        mediump vec3 lightDir = lightDirAccurate;
        lightDir *= inversesqrt(distSq);

        highp vec3 spotDir = lightDirectionsWorld[i];
        attenuation *= spotAttenuation(lightDir, spotDir, lightParameters[i].x, lightParameters[i].y);

        if(attenuation < 0.001)
            continue;

        mediump vec3 value = physicalBrdf(lightDir, normal, view, albedo.rgb, metallic, roughness, f0);

        float shadow = 1.0;
        #if NUM_SHADOWS > 0
        /* Shadows */
        bool shadowsEnabled = bool(lightParameters[i].z);
        if(shadowsEnabled) {
            int shadowIndex = int(lightParameters[i].w);
            shadow = sampleShadowPerspective(shadowIndex, surface.normal, lightDir);
        }
        #endif

        col += attenuation*value*lightData.rgb*shadow;
    }

    lowp uint endSunLights = numPointLights + numSpotLights + numSunLights;
    for(; i < endSunLights; ++i) {
        mediump vec4 lightData = lightColors[i];
        mediump vec3 lightDir = lightDirectionsWorld[i];

        mediump vec3 value = physicalBrdf(lightDir, normal, view, albedo.rgb, metallic, roughness, f0);

        float shadow = 1.0;
        #if NUM_SHADOWS > 0
        /* Shadows */
        bool shadowsEnabled = bool(lightParameters[i].z);
        if(shadowsEnabled) {
            int shadowIndex = int(lightParameters[i].w);
            float depth = -fragPositionView.z;
            int cascade = selectCascade(shadowIndex, depth);
            if(cascade != -1)
                shadow = sampleShadowOrtho(shadowIndex + cascade, surface.normal, lightDir);
        }
        #endif

        col += value*lightData.a*lightData.rgb*shadow;
    }

    #endif

    #ifdef WITH_EMISSIVE
    vec4 emissive = mat.emissiveColor;
    #ifdef TEXTURED
    if(mat.emissiveTexture != 0u) {
        emissive *= textureAtlas(mat.emissiveTexture, fragTextureCoords);
    }
    #endif
    col += emissive.a*srgbToLinear(emissive.rgb);
    #endif

    #ifdef TONEMAPPING
    col = col/(col + vec3(1.0));
    #endif

    outColor = linearToSrgb(vec4(col, albedo.a));
}
