/*
 * Set of functions you can re-use to work on colors.
 */

/**
 * @brief Create a gradient between four colors
 *
 * @param stop0 The color of the first stop point
 * @param stop1 The color of the second stop point
 * @param stop2 The color of the third stop point
 * @param stop3 The color of the fourth stop point
 *
 * This method assumes the stops are linear and spaced uniformly.
 */
mediump vec4 gradient4(mediump vec4 stop0, mediump vec4 stop1, mediump vec4 stop2, mediump vec4 stop3, highp float value) {
    highp float value2 = value * 2.0;
    vec4 a = mix(stop0, stop1, value2);
    vec4 b = mix(stop2, stop3, value2 - 1.0);
    return mix(a, b, smoothstep(0.495, 0.505, value));
}

/**
 * @brief Apply the sRGB transfer function to a linear RGB color
 *
 * @param linear Linear RGB color
 * @returns Non-linear sRGB color
 *
 * Uses a 2.2 gamma curve as a fast approximation for the sRGB EOTF. Alpha is
 * unaffected.
 */
lowp vec4 linearToSrgb(mediump vec4 linear) {
    return vec4(pow(linear.rgb, vec3(1.0/2.2)), linear.a);
}

/** @overload */
lowp vec3 linearToSrgb(mediump vec3 linear) {
    return pow(linear, vec3(1.0/2.2));
}

/**
 * @brief Apply the inverse sRGB transfer function to get a linear RGB color
 *
 * @param srgb Non-linear sRGB color
 * @returns Linear RGB color
 *
 * Uses a 2.2 gamma curve as a fast approximation for the sRGB EOTF. Alpha is
 * unaffected.
 */
mediump vec4 srgbToLinear(mediump vec4 srgb) {
    /* Input is mediump to avoid precision issues in pow() */
    return vec4(pow(srgb.rgb, vec3(2.2)), srgb.a);
}

/** @overload */
mediump vec3 srgbToLinear(mediump vec3 srgb) {
    return pow(srgb, vec3(2.2));
}
