/*
 * Set of functions dedicated to Global Illumination.
 *
 * Dependencies:
 *     - Textures.frag
 *     - CoordinateSystems.frag
 */

#define MAX_ENV_SPECULAR_MIPS (5.0 - 1.0)

vec3 fresnelSchlickRoughness(float cosTheta, vec3 F0, float roughness) {
    return F0 + (max(vec3(1.0 - roughness), F0) - F0) * pow(clamp(1.0 - cosTheta, 0.0, 1.0), 5.0);
}

/**
 * Taken from: https://www.unrealengine.com/en-US/blog/physically-based-shading-on-mobile
 */
vec3 EnvBRDFApprox(vec3 specularColor, mediump float roughness, mediump float NdotV) {
    const lowp vec4 c0 = vec4(-1, -0.0275, -0.572, 0.022);
    const lowp vec4 c1 = vec4(1, 0.0425, 1.04, -0.04);
    lowp vec4 r = roughness * c0 + c1;
    lowp float a004 = min( r.x * r.x, exp2( -9.28 * NdotV ) ) * r.x + r.y;
    lowp vec2 AB = vec2( -1.04, 1.04 ) * a004 + r.zw;
    return specularColor * AB.x + AB.y;
}

void computeEnvContribution(uint irradianceTexture, uint specularTexture, vec3 normal, vec3 reflected,
    float NdotV, float perceptualRoughness, float metallic, vec3 F0, inout vec3 indirectDiffuse, inout vec3 indirectSpecular) {

    /* Diffuse contribution. */
    vec2 uv = cartesianToEquirectangular(normal);
    vec3 F = fresnelSchlickRoughness(NdotV, F0, perceptualRoughness);
    vec3 kD = vec3(1.0) - F;
    kD *= 1.0 - metallic;
    indirectDiffuse = kD * textureAtlas(irradianceTexture, uv, 0).rgb;

    /* Specular contribution. */
    float mip = perceptualRoughness * MAX_ENV_SPECULAR_MIPS;
    float level0 = floor(mip);
    float level1 = min(MAX_ENV_SPECULAR_MIPS, level0 + 1.0);
    float blend = mip - level0;

    vec2 uvSpecular = cartesianToEquirectangular(reflected);
    vec3 prefilteredMip0 = textureAtlas(specularTexture, uvSpecular, int(level0)).rgb;
    vec3 prefilteredMip1 = textureAtlas(specularTexture, uvSpecular, int(level1)).rgb;
    vec3 prefilteredColor = mix(prefilteredMip0, prefilteredMip1, blend);
    indirectSpecular = prefilteredColor * EnvBRDFApprox(F, perceptualRoughness, NdotV);
}
